// -*- mode: C++; tab-width: 4; indent-tabs-mode: nil -*- (for GNU Emacs)
#ifndef _filter_h_
#define _filter_h_

/*--- some declarations from winsock.h ---*/

/*
 * Basic system type definitions, taken from the BSD file sys/types.h.
 */
typedef unsigned char   u_char;
typedef unsigned short  u_short;
typedef unsigned int    u_int;
typedef unsigned long   u_long;

// some socket functions
u_long ntohl (u_long netlong);
u_short ntohs (u_short netshort);

/*
 * Protocols
 */
#define IPPROTO_IP              0               /* dummy for IP */
#define IPPROTO_TCP             6               /* tcp */
#define IPPROTO_UDP             17              /* user datagram protocol */

/*
 * Structure used by kernel to store most
 * addresses.
 */
struct sockaddr {
        u_short sa_family;              /* address family */
        char    sa_data[14];            /* up to 14 bytes of direct address */
};

/*
 * Internet address (old style... should be updated)
 */
struct in_addr {
        union {
                struct { u_char s_b1,s_b2,s_b3,s_b4; } S_un_b;
                struct { u_short s_w1,s_w2; } S_un_w;
                u_long S_addr;
        } S_un;
#define s_addr  S_un.S_addr
                                /* can be used for most tcp & ip code */
#define s_host  S_un.S_un_b.s_b2
                                /* host on imp */
#define s_net   S_un.S_un_b.s_b1
                                /* network */
#define s_imp   S_un.S_un_w.s_w2
                                /* imp */
#define s_impno S_un.S_un_b.s_b4
                                /* imp # */
#define s_lh    S_un.S_un_b.s_b3
                                /* logical host */
};

/*
 * Socket address, internet style.
 */
struct sockaddr_in {
        short   sin_family;
        u_short sin_port;
        struct  in_addr sin_addr;
        char    sin_zero[8];
};

/*
 * Address families.
 */
#define AF_INET         2               /* internetwork: UDP, TCP, etc. */


/*----------------------------------------*/

/*
 * direction type for filter
 * for quick filter:
 *  if proto == IPPROTO_TCP (DIRECTION_IN - accept connections; DIRECTION_OUT - connect)
 *  if proto == IPPROTO_UDP (DIRECTION_IN - receive datagram; DIRECTION_OUT - send datagram)
 */
#define DIRECTION_IN    1
#define DIRECTION_OUT   2

// filter result
#define FILTER_ALLOW    1
#define FILTER_DENY     2
#define FILTER_CONTENT  3

// alloc callback fn
typedef void *alloc_fn_t(u_long size);

typedef void * cont_flt_t;

#define CONTFLT_ALLOWALL	((cont_flt_t)-1)
#define CONTFLT_DENYALL		((cont_flt_t)-2)

// quick filter
int quick_filter(const struct sockaddr *name_from, const struct sockaddr *name_to,
    int namelen, int proto, int direction, cont_flt_t *flt_in, cont_flt_t *flt_out);

// content filter
int content_filter(cont_flt_t flt,
	const char *data, u_long data_size, u_long data_offset,
	char **new_data, u_long *new_data_size, alloc_fn_t *alloc_fn, int is_oob);

char *memstr(const char *data, u_long data_size, const char *key, u_long key_size);

#endif
